--Ver:1.0

local function getParam(args, fieldName, fieldType)
    local param = nil
    if args ~= nil and args[fieldName] ~= nil then
        if fieldType == "integer" then
            param = args[fieldName]:intValue()
        elseif fieldType == "decimal" then
            param = args[fieldName]:floatValue()
        elseif fieldType == "string" then
            param = args[fieldName]:getCString()
        elseif fieldType == "boolean" then
            param = (args[fieldName]:getCString() == "true")
        else
            param = args[fieldName]
        end
    end
    return param
end

local function createPathTableAnimation(duration, pathTable, beginIndex, endIndex)
    local animation = CCAnimation:create()
    local action = nil
    
    local number = 1
    if beginIndex ~= nil then
        number = beginIndex
    end
    
    local index = ""..number
    while pathTable[index] ~= nil and (endIndex == nil or number <= endIndex) do
        local path = pathTable[index]:getCString()
        animation:addSpriteFrameWithFileName(path)
        number = number + 1
        index = ""..number
    end
    
    animation:setDelayPerUnit(duration /  (number - 1))
    animation:setRestoreOriginalFrame(true)
    action = CCAnimate:create(animation)
    return action
end

local function createFramePathAnimation(duration, framePath, frameSize, beginIndex, endIndex)
    local animation = CCAnimation:create()
    local action = nil

    if beginIndex == nil then
        beginIndex = 1
    end
    
    if endIndex == nil then
        endIndex = frameSize
    end
    
    for i = beginIndex, endIndex do
        if i < 10 then
            number = "0"..i
        else
            number = i
	    end
	    
        local name = framePath..number..".png"
        animation:addSpriteFrameWithFileName(name)
    end
    animation:setDelayPerUnit(duration / frameSize)
    animation:setRestoreOriginalFrame(true)
    action = CCAnimate:create(animation)
    return action
end

local function createImageTableAnimation(duration, imageTable, beginIndex, endIndex)
    local animation = CCAnimation:create()
    local action = nil
    
    local number = 1
    if beginIndex ~= nil then
        number = beginIndex
    end
    
    local index = ""..number
    while imageTable[index] ~= nil and (endIndex == nil or number <= endIndex) do
        local texture = imageTable[index]

        local size = texture:getContentSize()

        local rect = CCRectMake(0, 0, size.width, size.height)
        animation:addSpriteFrameWithTexture(texture, rect)
        number = number + 1
        index = ""..number
        --CCLuaLog("imageTable index:"..index);
    end
    animation:setDelayPerUnit(duration /  (number - 1))
    animation:setRestoreOriginalFrame(false)
    action = CCAnimate:create(animation)
    return action
end

function copyright_script_actionAnimate(args)
    local scope = args["scope"]
    local duration = getParam(args, "duration", "decimal")
	  local pathTable = args["pathTable"]
	  local imageTable = args["imageTable"]
    local frameSize = getParam(args, "framesize", "integer")
    local framePath = getParam(args, "framepath", "string")
    local isRepeat = getParam(args, "isRepeat", "string") --循环次数[once:一次|forever:无限循环|customLoop:自定义循环次数]
    local repeatCount = getParam(args, "repeatCount", "integer") --自定义循环次数
    local repeatStart = getParam(args, "repeatStart", "integer")
    local repeatEnd = getParam(args, "repeatEnd", "integer")
    local resetAfter = args["resetAfter"]

--判断是否正在运行动画
    local runaction1 =  scope:getActionByTag(23)
    if runaction1 ~=nil then
    return
    end
    local runaction2 =  scope:getActionByTag(24)
    if runaction2 ~=nil then
    return
    end
    --延迟
    local delay = args["delay"]
    local delayAction = nil
    if delay ~= nil then
        delayAction = CCDelayTime:create(delay:floatValue())
    end
    local action = nil
    local subAction = nil
	--scope:stopAllActions()
	  
    if pathTable ~= nil then --通过图片路径table来创建动画。
        if isRepeat == "once" or isRepeat == "false" then
            action = createPathTableAnimation(duration, pathTable, nil, nil)
        elseif isRepeat == "forever" or isRepeat == "true" then
            subAction = CCCallFunc:create(
                function()
                    local repeatAction = createPathTableAnimation(duration, pathTable, repeatStart, repeatEnd)
                    repeatAction = CCRepeatForever:create(repeatAction)
	                  scope:runAction(repeatAction)
                end
            )
        elseif isRepeat == "customLoop" then
            subAction = CCCallFunc:create(
                function()
                    local repeatAction = createPathTableAnimation(duration, pathTable, repeatStart, repeatEnd)
                    repeatAction = CCRepeat:create(repeatAction, repeatCount)
                    scope:runAction(repeatAction)
                end
            )
        end
    elseif framePath ~= nil and frameSize ~= nil then --通过图片路径前缀和帧数来创建动画
        if isRepeat == "once" or isRepeat == "false" then
            action = createFramePathAnimation(duration, framePath, frameSize, nil, nil)
        elseif isRepeat == "forever" or isRepeat == "true" then
            subAction = CCCallFunc:create(
                function()
                    local repeatAction = createFramePathAnimation(duration, framePath, frameSize, repeatStart, repeatEnd)
                    repeatAction = CCRepeatForever:create(repeatAction)
	                  scope:runAction(repeatAction)
                end
            )
        elseif isRepeat == "customLoop" then
            subAction = CCCallFunc:create(
                function()
                    local repeatAction = createFramePathAnimation(duration, framePath, frameSize, repeatStart, repeatEnd)
                    repeatAction = CCRepeat:create(repeatAction, repeatCount)
                    scope:runAction(repeatAction)
                end
            )
        end
    elseif imageTable ~= nil then --通过已经加载的图片table来创建动画
        if isRepeat == "once" or isRepeat == "false" then
            action = createImageTableAnimation(duration, imageTable, nil, nil)
        elseif isRepeat == "forever" or isRepeat == "true" then
            subAction = CCCallFunc:create(
                function()
                    local repeatAction = createImageTableAnimation(duration, imageTable, repeatStart, repeatEnd)
                    repeatAction = CCRepeatForever:create(repeatAction)
                    repeatAction:setTag(24)
	                  scope:runAction(repeatAction)
                end
            )
        elseif isRepeat == "customLoop" then
            subAction = CCCallFunc:create(
                function()
                    local repeatAction = createImageTableAnimation(duration, imageTable, repeatStart, repeatEnd)
                    repeatAction = CCRepeat:create(repeatAction, repeatCount)
                    repeatAction:setTag(24)
                    scope:runAction(repeatAction)
                end
            )
        end
    end
    
    local actionList = CCArray:create()

    if delayAction ~= nil then
       actionList:addObject(delayAction)
    end

    if action ~= nil then
        actionList:addObject(action)
    else
        actionList:addObject(subAction)
    end

--恢复原样
    if resetAfter ~= nil then
        if (isRepeat == "once" and resetAfter:getCString() == "true")
           or (isRepeat == "customLoop" and resetAfter:getCString() == "true")
        then
            local state = scope:getTexture()
            local resetAfterAction = CCCallFunc:create(
                function()
                    --恢复
                    scope:setTexture(state)
                end
            )
            actionList:addObject(resetAfterAction)
        end
    end

    if actionList:count() ~= 0 then
        local actionSeq = CCSequence:create(actionList)
        actionSeq:setTag(23)
        scope:runAction(actionSeq)
    end
--[[
    if actionList:count() ~= 0 then
        local actionSeq = CCSequence:create(actionList)
        scope:runAction(actionSeq)
    end
--]]
end



